#!/bin/sh

# On flx01:
#    cd /data/public/master.formilux.org/git/dist
#    find * -type d -name '*.git' > repos.lst
#
#    for repo in $(cat repos.lst ); do
#      GIT_DIR=$repo git-show-ref --heads --tags |sed -e "s,^,${repo%.git} ,"
#    done >refs.lst
#

cd /formilux
mkdir git
cd git

# URL for different protocols, without the trailing slash
MASTER_HTTP=http://master.formilux.org/git
MASTER_SSH=ssh://master.formilux.org/git
MASTER_GIT=git://master.formilux.org/git

# Use the git protocol by default
MASTER=$MASTER_HTTP
ORIGIN=origin

# Create other remotes sites for explicit protocol variants, with names such
# as "origin-git", "origin-ssh", "origin-http". However, we want them to share
# the same local copy of the remote branch so that we can fetch/push from/to
# either, so we add them by hand.

do_create_all_proto() {
	url="$(git config --get remote.$ORIGIN.url)"
	url="${url#$MASTER}"
	spec="$(git config --get remote.$ORIGIN.fetch)"

	git remote add $ORIGIN-git "${MASTER_GIT}${url}" &&
		git config remote.$ORIGIN-git.fetch "${spec}"

	git remote add $ORIGIN-ssh "${MASTER_SSH}${url}" &&
		git config remote.$ORIGIN-ssh.fetch "${spec}"

	git remote add $ORIGIN-http "${MASTER_HTTP}${url}" &&
		git config remote.$ORIGIN-http.fetch "${spec}"
}

# Create all branches that appear in the origin remote and set them up to
# track the origin by default.

do_create_all_branches() {
	for branch in $(git branch -r); do
		name="${branch#$ORIGIN/}"
		if [ -z "$name" -o "$name" == "master" -o "$name" = "HEAD" ]; then
			continue
		fi
		echo "Creating branch $name"
		git branch --no-track $name $ORIGIN/$name &&
			git config branch.$name.remote $ORIGIN &&
			git config branch.$name.merge refs/heads/$name
	done
}


######### main

# get the list of repositories
lynx -source $MASTER_HTTP/dist/repos.lst > repos.lst

# clone original package and track it under remote "origin"
for repo in $(cat repos.lst); do
	dir=${repo%.git}
	git clone -o $ORIGIN $MASTER/dist/$repo ${repo%.git}
	( cd $dir; do_create_all_proto; do_create_all_branches; )
done

exit 0

#### to update the refs list on the client :
#### for repo in $(cat repos.lst ); do GIT_DIR=${repo%.git}/.git git-show-ref --heads --tags |sed -e "s,^,${repo%.git} ,";done >refs.lst

#### to update the remote refs list on the client :
#### for repo in $(cat repos.lst ); do
####    GIT_DIR=${repo%.git}/.git git-show-ref |
####       sed -e '/refs\/heads\//d' \
####           -e '/refs\/remotes\/origin\//s,refs/remotes/origin,refs/heads,' \
####           -e '/refs\/heads\/HEAD$/d' | sed -e "s,^,${repo%.git} ,"
#### done >refs.lst

#### for repo in $(cat repos.lst ); do GIT_DIR=${repo%.git}/.git git-show-ref --heads --tags |sed -e "s,^,${repo%.git} ,";done >refs.lst
#### for repo in $(cat repos.lst ); do GIT_DIR=${repo%.git}/.git git-show-ref --heads --tags |sed -e "s,^,${repo%.git} ,";done >refs.lst

#### to update the refs list on the server :
#### for repo in $(cat repos.lst ); do GIT_DIR=$repo git-show-ref --heads --tags |sed -e "s,^,${repo%.git} ,";done >refs.lst


#### to recreate a bare repo from a local repo :

git clone --mirror pkg/automake blabla.git
cd blabla.git
git init --bare --shared=group
git config --remove-section remote.origin
rm -f info/refs
grep -vF 'refs/remotes/origin/' packed-refs > packed-refs~ && mv packed-refs~ packed-refs
git gc --aggressive

