#!/bin/sh

if [ -z "$1" -o "$1" == "-h" -o "$1" == "--help" ]; then
	echo "Usage: ${0##*/} <git_repository> [-]"
	echo "Will create ./<git_repository>.git.tgz, unless '-' is specified, in"
	echo "which case it will send the gzipped stream to standard output."
	exit 1
fi >&2

if [ ! -d "$1/." ]; then
	echo "Fatal: $1 is not a directory."
	exit 2
fi >&2

CUR="$PWD"
DIR="${1%/}"
DIR="${DIR##*/}"
DIR="${DIR%.git}.git"
TMP="/tmp/$$.$RANDOM"

if ! mkdir "$TMP"; then
	echo "Fatal: cannot create temporary directories $TMP"
	exit 3
fi

if ! git clone --mirror "$1" "$TMP/$DIR"; then
	echo "Fatal: cannot clone $1 into $TMP/$DIR"
	rmdir "$TMP/$DIR" "$TMP" 2>/dev/null
	exit 4
fi

export GIT_DIR="$TMP/$DIR"
cd "$GIT_DIR"
git init --bare --shared=group
find . -type d | xargs chmod g+ws

git config --remove-section remote.origin
rm -f "$GIT_DIR/info/refs"
grep -vF 'refs/remotes/origin/' "$GIT_DIR/packed-refs" > "$GIT_DIR/packed-refs~"
mv "$GIT_DIR/packed-refs~" "$GIT_DIR/packed-refs"

mv "$GIT_DIR/hooks/post-update.sample" "$GIT_DIR/hooks/post-update" 2>/dev/null
git gc --aggressive

cd "$TMP"
if [ "$2" == "-" ]; then
	tar cf - "$DIR" | gzip -c9
else
	tar cf - "$DIR" | gzip -c9 > "$CUR/$DIR.tgz"
fi

cd $CUR
rm -rf "$TMP"

echo "Done."
ls -la "$DIR.tgz"
