#!/bin/sh

if [ -z "$2" -o "$1" == "-h" -o "$1" == "--help" ]; then
	echo "Usage: ${0##*/} <local_git_repository> <path_on_master>"
	echo "Will setup local git repo to track master's  package in path_on_master."
	echo "Eg: ${0##*/} init-scripts /git/dist/pkg/init-scripts.git"
	exit 1
fi >&2

if [ ! -d "$1/." ]; then
	echo "Fatal: $1 is not a directory."
	exit 2
fi >&2

# URL for different protocols, without the trailing slash
MASTER_HTTP=http://master.formilux.org
MASTER_SSH=ssh://master.formilux.org
MASTER_GIT=git://master.formilux.org

# Use the git protocol by default
MASTER=$MASTER_HTTP
ORIGIN=origin

# Create other remotes sites for explicit protocol variants, with names such
# as "origin-git", "origin-ssh", "origin-http". However, we want them to share
# the same local copy of the remote branch so that we can fetch/push from/to
# either, so we add them by hand.

do_create_all_proto() {
	url="$(git config --get remote.$ORIGIN.url)"
	url="${url#$MASTER}"
	spec="$(git config --get remote.$ORIGIN.fetch)"

	git remote add $ORIGIN-git "${MASTER_GIT}${url}" &&
		git config remote.$ORIGIN-git.fetch "${spec}"

	git remote add $ORIGIN-ssh "${MASTER_SSH}${url}" &&
		git config remote.$ORIGIN-ssh.fetch "${spec}"

	git remote add $ORIGIN-http "${MASTER_HTTP}${url}" &&
		git config remote.$ORIGIN-http.fetch "${spec}"
}

# Automatically track all local branches from the origin.

do_track_all_branches() {
	for branch in $(git-show-ref --heads |sed -e 's,^.*refs/heads/,,'); do
		name="${branch#$ORIGIN/}"
		if [ -z "$name" -o "$name" == "master" -o "$name" = "HEAD" ]; then
			continue
		fi
		echo "Updating branch $name"
		git branch --no-track $name $ORIGIN/$name 2>/dev/null
		git config branch.$name.remote $ORIGIN &&
			git config branch.$name.merge refs/heads/$name
	done
}

#### main

CUR="$PWD"
DIR="${1%/}"
[ -z "${DIR}" -o -n "${DIR##/*}" ] && DIR="$CUR/$DIR"

POM="${2%/}"
POM="${POM#*://}"

cd "$DIR"

# remove possibly existing previous origins without removing
# tracking branches.
git config --remove-section remote.${ORIGIN}      2>/dev/null
git config --remove-section remote.${ORIGIN}-ssh  2>/dev/null
git config --remove-section remote.${ORIGIN}-git  2>/dev/null
git config --remove-section remote.${ORIGIN}-http 2>/dev/null

rm -f ".git/info/refs"
[ -d ".git/refs/remotes/$ORIGIN/." ] && rm -rf ".git/refs/remotes/$ORIGIN"

if [ -s ".git/packed-refs" ]; then
	grep -vF "refs/remotes/$ORIGIN/" ".git/packed-refs" > \
	  ".git/packed-refs~" && mv ".git/packed-refs~" ".git/packed-refs"
	grep -vF "refs/remotes/$ORIGIN-ssh/" ".git/packed-refs" > \
	  ".git/packed-refs~" && mv ".git/packed-refs~" ".git/packed-refs"
	grep -vF "refs/remotes/$ORIGIN-git/" ".git/packed-refs" > \
	  ".git/packed-refs~" && mv ".git/packed-refs~" ".git/packed-refs"
	grep -vF "refs/remotes/$ORIGIN-http/" ".git/packed-refs" > \
	  ".git/packed-refs~" && mv ".git/packed-refs~" ".git/packed-refs"
fi
git gc --aggressive

# create the "origin" remote
git remote add $ORIGIN "${MASTER}${POM}"
do_create_all_proto
do_track_all_branches

# OK that's finished.
cd "$CUR"

exit 0
######
###### note that when we want to create a new branch, here's what we have to do :
######    git branch foo
######
######  then later:
######    git push origin-ssh foo
######  note that the post-update hook must be OK on the other side
######    git config branch.foo.remote origin
######    git config branch.foo.merge  refs/heads/foo
